from django import template

def chunk(chunkable, size=2):
    chunkable = list(chunkable)
    counter = 0
    chunked = []
    sublist = []
    while len(chunkable):
        for i in range(size):
            sublist.append(chunkable.pop(0))
        chunked.append(sublist)
        sublist = []
    return chunked

class TemplateTagNode(template.Node):

    noun_for = {'by': 'who'}

    @classmethod
    def error_msg(cls):
        parts = []
        for preposition, noun in cls.noun_for.items():
            parts.append("%s <%s>" % (preposition, noun))
        parts = ' '.join(parts)

        msg = """Invalid tag syntax "%s"
Syntax: %s """ + parts + """ as <varname>"""
        return msg

    @classmethod
    def process_tag(cls, parser, token):
        words = token.split_contents()
        tag_name, words = words[0], words[1:]

        _as, varname = words[-2:]
        words = words[:-2]

        if _as != 'as':
            raise template.TemplateSyntaxError, \
                cls.error_msg() % (token.contents, tag_name)
        
        words = chunk(words)
        if len(words) != len(cls.noun_for):
            raise template.TemplateSyntaxError, \
                cls.error_msg() % (token.contents, tag_name)
    
        kw = {}
        for phrase in words:
            preposition, noun = phrase
            if preposition not in cls.noun_for \
                    or cls.noun_for[preposition] in kw:
                raise template.TemplateSyntaxError, \
                    cls.error_msg() % (token.contents, tag_name)
            kw[cls.noun_for[preposition]] = noun

        return cls(varname, **kw)

    def __init__(self, varname, **kw):
        self.varname = varname
        self.vars = {}
        for key, var in kw.items():
            self.vars[key] = template.Variable(var)

    def render(self, context):
        vars = dict(self.vars)
        for key, var in vars.items():
            vars[key] = var.resolve(context)

        context[self.varname] = self.execute_query(**vars)
        return ''
    
    def execute_query(self, **kw):
        return ''



from django import template
from djangohelpers.templatetags import TemplateTagNode

register = template.Library()

class ReplaceValue(TemplateTagNode):
    """
    Copies a dict with a value swapped; for example:
      >> my_dict = {'a': 1, 'b': 2, 'c': 3}
      >> {% replace_value of 'b' with 7 in my_dict as new_dict %}
      >> new_dict
      {'a': 1, 'b': 7, 'c': 3}

    Set a value to the empty string to silently delete it:
      >> {% replace_value of 'b' with '' in my_dict as new_dict %}
      >> new_dict
      {'a': 1, 'c': 3}
    """
    noun_for = {"of":"key", "in":"items", "with":"value"}

    def __init__(self, varname, key, items, value):
        TemplateTagNode.__init__(self, varname, key=key, items=items, value=value)

    def execute_query(self, key, items, value):
        new = dict(items)
        if value == '':
            del new[key]
        else:
            new[key] = value
        return new
register.tag('replace_value', ReplaceValue.process_tag)

def qsify(_dict):
    """
    converts a dict into a query string:
     {{my_dict|qsify}}
    """
    qs = '?'
    try:
        iterator = _dict.iterlists()
    except AttributeError:
        iterator = _dict.iteritems()

    for key, value in iterator:
        if isinstance(value, list):
            for v in value:
                qs += '%s=%s&' % (key, v)
        else:
            qs += '%s=%s&' % (key, value)
    qs=qs.rstrip("&")
    return qs
register.filter('qsify', qsify)

def split(text):
    """ sometext|split """
    return text.split()
register.filter('split', split)

def getitem(dict, item):
    """ my_dict|getitem:'b' """
    try:
        return dict[item]
    except KeyError:
        return ''
register.filter('getitem', getitem)

def lessthan(a, b):
    return a < b
register.filter('lessthan', lessthan)

def greaterthan(a, b):
    return a > b
register.filter('greaterthan', greaterthan)



from setuptools import setup, find_packages
import sys, os

version = '0.5'

long_description = open('README.txt').read()

setup(name='djangohelpers',
      version=version,
      description="a collection of useful middleware, template tags, etc",
      long_description=long_description,
      classifiers=[], 
      keywords='',
      author='Ethan Jucovy',
      author_email='ejucovy@gmail.com',
      url='',
      license='GPL',
      packages=find_packages(exclude=['ez_setup', 'examples', 'tests']),
      include_package_data=True,
      zip_safe=False,
      install_requires=[
      ],
      entry_points="""
      [console_scripts]
      maketemplatetag = djangohelpers.console_scripts:main
      """,
      )
